#!/bin/sh -x

#This script upgrades the
#	1. Extract and verify the firmware.
#	1. Copy the sysdisk.img.gz to NAND.
#	2. Write kernel to NOR flash.
#	3. Update checksum in NAND.

ECHO="/bin/echo"
#FLASH_ERASE="/sbin/flash_erase"
ENV_PARTITION="/dev/mtd2"
KEY_PATH="nas3xx_fw"
USB_PATH="/mnt/partnerkey"
USB_FW_PATH="${USB_PATH}/${KEY_PATH}"
BUFFER_PATH="${USB_PATH}/buffer"
MD5SUM_FILE="${USB_PATH}/nas3xx_md5sum.txt"

FW_PATH="/firmware"
INFO_PRINTENV="${FW_PATH}/sbin/fw_printenv"
INFO_SETENV="${FW_PATH}/sbin/fw_setenv"
FW_FILE="${USB_FW_PATH}/ras.bin"
RESET_FLAG="${USB_FW_PATH}/RESET_CONFIG"	# load default
RESET_MAC="${USB_FW_PATH}/RESET_MAC"		# reset MAC address

UBOOT_FILE="${USB_FW_PATH}/uboot.bin"
UBOOT_MD5SUM="5fcbace4bbe2042d92a2e3dcc8dfe630"
UBOOT_TIME_STAMP="Oct 13 2015 - 09:25:56"


# path to erase, mount, and write
NANDPATH="${FW_PATH}/mnt/nand2"

# which kernel and system image partitions
CURR_BOOTFROM=`${INFO_PRINTENV} curr_bootfrom | awk -F"=" '{print $2}'`
if [ ${CURR_BOOTFROM} -eq 1 ]; then
	NEXT_BOOTFROM=2
elif [ ${CURR_BOOTFROM} -eq 2 ]; then
	NEXT_BOOTFROM=1
else
	echo "ERROR!!! Invalid INFO value 'curr_bootfrom'!"
	# All leds show red
	setLED SYS RED BLINK
	setLED HD RED BLINK
	setLED COPY RED BLINK
	exit 1
fi


CURR_KERNEL_MTD=`${INFO_PRINTENV} kernel_mtd_${CURR_BOOTFROM} | awk -F"=" '{print $2}'`
CURR_IMG_MTD=`${INFO_PRINTENV} sysimg_mtd_${CURR_BOOTFROM} | awk -F"=" '{print $2}'`

NEXT_KERNEL_MTD=`${INFO_PRINTENV} kernel_mtd_${NEXT_BOOTFROM} | awk -F"=" '{print $2}'`
NEXT_IMG_MTD=`${INFO_PRINTENV} sysimg_mtd_${NEXT_BOOTFROM} | awk -F"=" '{print $2}'`

md5sum -c ${MD5SUM_FILE}
if [ $? -ne 0 ]; then
	echo "Key is corrupt"
	exit 1
fi


error_exit()
{
	# All leds show red
	# setLED SYS RED BLINK
	# setLED HDD1 RED BLINK
	# setLED HDD2 RED BLINK
	# setLED HDD3 RED BLINK
	# setLED HDD4 RED BLINK
	# setLED COPY RED BLINK
	setLED SYS OFF
	setLED HDD1 OFF
	setLED HDD2 OFF
	setLED USB OFF
	setLED COPY OFF
	setLED SYS RED ON
	setLED HDD1 RED ON
	setLED HDD2 RED ON
	setLED COPY RED ON
	read	# keep current status
}

load_default_config()
{
	rm -rf /i-data/.media/twonkymedia
	rm -rf /etc/zyxel/conf
	rm -rf /etc/zyxel/storage/*
	umount /etc/zyxel
	ubirmvol /dev/ubi3 -N ubi_config
	ubidetach -m 3
	rm -rf /etc/zyxel/
}

update_uboot()
{
	echo "updating u-boot ..."
	nanddump /dev/mtd0 | grep "${UBOOT_TIME_STAMP}"
	if [ "$?" = "0" ]; then
		echo "*** the u-boot installed has been already the latest one, no need to update now. ***"
		return
	fi

	UBOOT_CAL_MD5SUM="`md5sum ${UBOOT_FILE} | awk '{print $1}'`"
	if [ "${UBOOT_MD5SUM}" != "${UBOOT_CAL_MD5SUM}" ]; then
		echo "wrong md5sum. ${UBOOT_FILE} may be corrupt."
	else
		flash_erase /dev/mtd0 0 0
		nandwrite -a -p /dev/mtd0 ${UBOOT_FILE}
		nanddump /dev/mtd0 > /tmp/uboot.bin
		strings /tmp/uboot.bin | grep "${UBOOT_TIME_STAMP}"
		if [ "$?" != "0" ]; then
			echo "the u-boot are written failed!!! this device may corrupt now!!!"
			error_exit
		fi
	fi
}


# disable power button
#/bin/echo "0" > /proc/shutdownStatus

# SYS LED blinks orange
/sbin/setLED SYS ORANGE BLINK
sleep 1

# Using the USB KEY as a temparory buffer
mkdir -p ${BUFFER_PATH} > /dev/null 2>&1

/sbin/bin2ram little "${FW_FILE}" ${BUFFER_PATH}/tlv.bin > /dev/null 2>&1
# ROMBIN is 0x04, firmware package type
if [ $? -ne 4 ]; then
	echo "Fail to extract FW file!"
	error_exit
fi
sync
sleep 1

#/sbin/fw_unpack -C ${BUFFER_PATH} -i ${BUFFER_PATH}/tlv.bin > /dev/null 2>&1
#/sbin/fw_unpack -C ${BUFFER_PATH} -i ${BUFFER_PATH}/tlv.bin 
cd ${BUFFER_PATH}
/sbin/fw_unpack -C . -i ./tlv.bin 
if [ $? -ne 0 ]; then
	echo "Fail to unpack FW file!"
	error_exit
fi
sync
sync
ls ${BUFFER_PATH}
# DATA_0000: header version
# DATA_0001: firmware version
# DATA_0002: firmware revision
# DATA_0101: model number 1
# DATA_0102: model number 2
# DATA_0200: core checksum
# DATA_0201: ZLD checksum; in NSA-310, it is the md5sum of sysdisk.img.gz
# DATA_0202: ROM checksum
# DATA_0203: InitRD checksum; there is no InitRD in NSA-310
# DATA_0204: IMG checksum; in NSA-310, it is the md5sum of sysdisk.img
# DATA_1000: kernel file, uImage
# DATA_1002: InitRD image, initrd.img.gz
# DATA_1004: System disk image, sysdisk.img.gz
# DATA_a000: executable, for some jobs before firmware upgrade
# DATA_a002: executable, for some jobs after firmware upgrade
sleep 10

/bin/rm -f ${BUFFER_PATH}/tlv.bin > /dev/null 2>&1

# check model number first
if [ -s ${BUFFER_PATH}/DATA_0101 ]; then
	board_model=`${FW_PATH}/sbin/mrd_model -p`
	file_model=`cat ${BUFFER_PATH}/DATA_0101`	
	echo -n "board_model=(${board_model}), file_model=(${file_model}) ... "

	if [ "x${board_model}" == "x${file_model}" ]; then
		echo "equal!"
	else
		echo "NOT equal! ${FW_PATH}/sbin/mrd_model -s ${file_model}"
		#force upgrade
		${FW_PATH}/sbin/mrd_model -s ${file_model}
		#error_exit
	fi
else
	echo "No model number in firmware file, skip upgrade procedure."
	error_exit
fi

# pre-process action (pre-install)
if [ -s ${BUFFER_PATH}/DATA_a000 ]; then
	chmod 755 ${BUFFER_PATH}/DATA_a000
	${BUFFER_PATH}/DATA_a000
	rm -rf ${BUFFER_PATH}/DATA_a000
fi

# kernel checksum verification
if [ -f ${BUFFER_PATH}/DATA_1000 ]; then
	CORE_MD5=`md5sum ${BUFFER_PATH}/DATA_1000 | awk '{print $1}'`
	CORE_FILE=`cat ${BUFFER_PATH}/DATA_0200`

	if [ "${CORE_MD5}" == "${CORE_FILE}" ]; then
		echo "Kernel checksum pass!"
	else
		echo "Kernel checksum MISMATCH!"
		error_exit
	fi
else
	echo "No kernel checksum file, skip upgrade procedure."
	error_exit
fi

# system image checksum verification
if [ -f ${BUFFER_PATH}/DATA_1004 ]; then
	ZLD_MD5=`md5sum ${BUFFER_PATH}/DATA_1004 | awk '{print $1}'`
	ZLD_FILE=`cat ${BUFFER_PATH}/DATA_0201`

	if [ "${CORE_MD5}" == "${CORE_FILE}" ]; then
		echo "System image checksum pass!"
	else
		echo "System image checksum MISMATCH!"
		error_exit
	fi
else
	echo "No system imgage checksum file, skip upgrade procedure."
	error_exit
fi


# umount config/rootfs partitions
/bin/umount `cat /proc/mounts|grep ubi|awk '{print $2}'` 2>/dev/null
ubidetach -m 3

/bin/umount /dev/mtdblock${CURR_IMG_MTD} > /dev/null 2>&1
/bin/umount /dev/mtdblock${NEXT_IMG_MTD} > /dev/null 2>&1


#umount /firmware/mnt/nand
# check if /dev/ubi_ctrl exists
ls -l /dev/ubi*
if [ ! -e /dev/ubi_ctrl ]; then
	${ECHO} -e "\033[033mCreating /dev/ubi_ctrl ...\033[0m"
	M="`cat /sys/class/misc/ubi_ctrl/dev | awk -F':' '{print $1}'`"
	m="`cat /sys/class/misc/ubi_ctrl/dev | awk -F':' '{print $2}'`"
	mknod /dev/ubi_ctrl c $M $m
	if [ "$?" != "0" ]; then
		${ECHO} "failed."
		error_exit
	fi
fi
ubidetach -d 0
ubidetach -m ${CURR_IMG_MTD}
ubidetach -m ${NEXT_IMG_MTD}


# upgrade system image
if [ -s ${BUFFER_PATH}/DATA_1004 ]; then
	/bin/mkdir -p ${NANDPATH}

	# Mount spare sysdisk img partition
	flash_erase /dev/mtd${NEXT_IMG_MTD} 0 0 > /dev/null 2>&1
	ubiattach -m ${NEXT_IMG_MTD} -d ${NEXT_IMG_MTD}
	if [ ! -e /dev/ubi${NEXT_IMG_MTD} ]; then
		M="`cat /sys/class/ubi/ubi${NEXT_IMG_MTD}/dev | awk -F':' '{print $1}'`"
		m="`cat /sys/class/ubi/ubi${NEXT_IMG_MTD}/dev | awk -F':' '{print $2}'`"
		mknod /dev/ubi${NEXT_IMG_MTD} c $M $m
		if [ "$?" != "0" ]; then
			${ECHO} "failed."
			error_exit
		fi
	fi
	ubimkvol /dev/ubi${NEXT_IMG_MTD} -N ubi_rootfs${NEXT_BOOTFROM} -m	# -s 100MiB
	mount -t ubifs ubi${NEXT_IMG_MTD}:ubi_rootfs${NEXT_BOOTFROM} ${NANDPATH}

	/bin/cp ${BUFFER_PATH}/DATA_1004 ${NANDPATH}/sysdisk.img.gz
	/bin/umount ${NANDPATH} > /dev/null 2>&1
	/bin/sync > /dev/null 2>&1

	# Mount current booting sysdisk img partition
	flash_erase /dev/mtd${CURR_IMG_MTD} 0 0 > /dev/null 2>&1
	ubiattach -m ${CURR_IMG_MTD} -d ${CURR_IMG_MTD}
	if [ ! -e /dev/ubi${CURR_IMG_MTD} ]; then
		M="`cat /sys/class/ubi/ubi${CURR_IMG_MTD}/dev | awk -F':' '{print $1}'`"
		m="`cat /sys/class/ubi/ubi${CURR_IMG_MTD}/dev | awk -F':' '{print $2}'`"
		mknod /dev/ubi${CURR_IMG_MTD} c $M $m
		if [ "$?" != "0" ]; then
			${ECHO} "failed."
			error_exit
		fi
	fi
	ubimkvol /dev/ubi${CURR_IMG_MTD} -N ubi_rootfs${CURR_BOOTFROM} -m	# -s 100MiB
	mount -t ubifs ubi${CURR_IMG_MTD}:ubi_rootfs${CURR_BOOTFROM} ${NANDPATH}

	/bin/cp ${BUFFER_PATH}/DATA_1004 ${NANDPATH}/sysdisk.img.gz
	/bin/umount ${NANDPATH} > /dev/null 2>&1
	/bin/sync > /dev/null 2>&1

fi


# upgrade kernel
if [ -s ${BUFFER_PATH}/DATA_1000 ]; then
	# Mount spare kernel partition
	flash_erase /dev/mtd${NEXT_KERNEL_MTD} 0 0 > /dev/null 2>&1
	/sbin/nandwrite -a -p /dev/mtd${NEXT_KERNEL_MTD} ${BUFFER_PATH}/DATA_1000

	nandwrite_result=$?
	nandwrite_retry=0

	while [ "${nandwrite_result}" != "0" -a ${nandwrite_retry} != "3" ]
	do
		flash_erase /dev/mtd${NEXT_KERNEL_MTD} 0 0 > /dev/null 2>&1
		/sbin/nandwrite -a -p /dev/mtd${NEXT_KERNEL_MTD} ${BUFFER_PATH}/DATA_1000
		nandwrite_result=$?
		nandwrite_retry=$(($nandwrite_retry+1))
	done

	if [ "${nandwrite_result}" != "0" ]; then
		echo "Fail to write kernel to nand"
		error_exit
	fi
	/bin/sync > /dev/null 2>&1

	# Mount current booting kernel partition
	flash_erase /dev/mtd${CURR_KERNEL_MTD} 0 0
	/sbin/nandwrite -a -p /dev/mtd${CURR_KERNEL_MTD} ${BUFFER_PATH}/DATA_1000
	nandwrite_result=$?
	nandwrite_retry=0

	while [ "${nandwrite_result}" != "0" -a ${nandwrite_retry} != "3" ]
	do
		flash_erase /dev/mtd${CURR_KERNEL_MTD} 0 0
		/sbin/nandwrite -a -p /dev/mtd${CURR_KERNEL_MTD} ${BUFFER_PATH}/DATA_1000
		nandwrite_result=$?
		nandwrite_retry=$(($nandwrite_retry+1))
	done

	if [ "${nandwrite_result}" != "0" ]; then
		echo "Fail to write kernel to nand"
		error_exit
	fi
	/bin/sync > /dev/null 2>&1
fi


# update checksums
if [ -s ${BUFFER_PATH}/DATA_0001 ]; then
	${INFO_SETENV} fwversion_${CURR_BOOTFROM} `/bin/cat ${BUFFER_PATH}/DATA_0001`
	${INFO_SETENV} fwversion_${NEXT_BOOTFROM} `/bin/cat ${BUFFER_PATH}/DATA_0001`
else
	${INFO_SETENV} fwversion_${CURR_BOOTFROM}
	${INFO_SETENV} fwversion_${NEXT_BOOTFROM}
fi

if [ -s ${BUFFER_PATH}/DATA_0002 ]; then
	${INFO_SETENV} revision_${CURR_BOOTFROM} `/bin/cat ${BUFFER_PATH}/DATA_0002`
	${INFO_SETENV} revision_${NEXT_BOOTFROM} `/bin/cat ${BUFFER_PATH}/DATA_0002`
else
	${INFO_SETENV} revision_${CURR_BOOTFROM}
	${INFO_SETENV} revision_${NEXT_BOOTFROM}
fi

if [ -s ${BUFFER_PATH}/DATA_0101 ]; then
	${INFO_SETENV} modelid_${CURR_BOOTFROM} `/bin/cat ${BUFFER_PATH}/DATA_0101`
	${INFO_SETENV} modelid_${NEXT_BOOTFROM} `/bin/cat ${BUFFER_PATH}/DATA_0101`
else
	${INFO_SETENV} modelid_${CURR_BOOTFROM}
	${INFO_SETENV} modelid_${NEXT_BOOTFROM}
fi

if [ -s ${BUFFER_PATH}/DATA_0200 ]; then
	${INFO_SETENV} core_checksum_${CURR_BOOTFROM} `/bin/cat ${BUFFER_PATH}/DATA_0200`
	${INFO_SETENV} core_checksum_${NEXT_BOOTFROM} `/bin/cat ${BUFFER_PATH}/DATA_0200`
else
	${INFO_SETENV} core_checksum_${CURR_BOOTFROM}
	${INFO_SETENV} core_checksum_${NEXT_BOOTFROM}
fi

if [ -s ${BUFFER_PATH}/DATA_0201 ]; then
	${INFO_SETENV} zld_checksum_${CURR_BOOTFROM} `/bin/cat ${BUFFER_PATH}/DATA_0201`
	${INFO_SETENV} zld_checksum_${NEXT_BOOTFROM} `/bin/cat ${BUFFER_PATH}/DATA_0201`
else
	${INFO_SETENV} zld_checksum_${CURR_BOOTFROM}
	${INFO_SETENV} zld_checksum_${NEXT_BOOTFROM}
fi

if [ -s ${BUFFER_PATH}/DATA_0202 ]; then
	${INFO_SETENV} romfile_checksum_${CURR_BOOTFROM} `/bin/cat ${BUFFER_PATH}/DATA_0202`
	${INFO_SETENV} romfile_checksum_${NEXT_BOOTFROM} `/bin/cat ${BUFFER_PATH}/DATA_0202`
else
	${INFO_SETENV} romfile_checksum_${CURR_BOOTFROM} `/bin/cat ${BUFFER_PATH}/DATA_0202`
	${INFO_SETENV} romfile_checksum_${NEXT_BOOTFROM} `/bin/cat ${BUFFER_PATH}/DATA_0202`
fi

if [ -s ${BUFFER_PATH}/DATA_0204 ]; then
	${INFO_SETENV} img_checksum_${CURR_BOOTFROM} `/bin/cat ${BUFFER_PATH}/DATA_0204`
	${INFO_SETENV} img_checksum_${NEXT_BOOTFROM} `/bin/cat ${BUFFER_PATH}/DATA_0204`
else
	${INFO_SETENV} img_checksum_${CURR_BOOTFROM}
	${INFO_SETENV} img_checksum_${NEXT_BOOTFROM}
fi

# post-process action
if [ -s ${BUFFER_PATH}/DATA_a002 ]; then
	chmod 755 ${BUFFER_PATH}/DATA_a002
	${BUFFER_PATH}/DATA_a002
	rm -rf ${BUFFER_PATH}/DATA_a002
fi

# load default if needed
if [ -f ${RESET_FLAG} ]; then
	load_default_config
fi

if [ -f ${RESET_MAC} ]; then
	# reset MAC
	${USB_FW_PATH}/mrd_mac 00 19 CB 00 51 81
fi

# update u-boot if needed
if [ -f ${UBOOT_FILE} ]; then
	update_uboot
fi


# enable power button
#/bin/echo "1" > /proc/shutdownStatus
sleep 1

/bin/sync

#/sbin/halt -f
echo "echo BYE" > /etc/init.d/rc.shutdown
/sbin/poweroff
#devmem 0xf1018130 32 0x00100000
sleep 6
read a

exit 0

